
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: BigTipDB, ChatFrame1, GameTooltip, GameTooltipDefaultContainer, IsAddOnLoaded, math, NamePlateTooltip, SixUI_TooltipAnchor, table, TinyTooltip, type

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Integration TinyTooltip")
local LibEvent = LibStub:GetLibrary("LibEvent.7000", true)

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

-- Check if the default position of the game tooltip container has been moved.
function M.HasGameTooltipDefaultContainerMoved()
	local point, relativeTo, relativePoint, x, y = GameTooltipDefaultContainer:GetPoint()

	return point ~= "TOPLEFT"
		or relativeTo ~= ChatFrame1
		or relativePoint ~= "TOPRIGHT"
		or math.modf(x) ~= 30
		or math.modf(y) ~= 10
end

function M:OnInitialize()
	if (not LibEvent) then return end
	LibEvent:attachTrigger("tooltip:hide", M.OnTooltipHide)
	LibEvent:attachTrigger("tooltip:show", M.OnTooltipShow)
	LibEvent:attachTrigger("tooltip.anchor.cursor.right", M.OnTooltipAnchor)
	LibEvent:attachTrigger("tooltip.anchor.cursor", M.OnTooltipAnchor)
	LibEvent:attachTrigger("tooltip.anchor.static", M.OnTooltipAnchor)
	LibEvent:attachTrigger("tooltip.style.background", M.OnTooltipStyleBackground)
	LibEvent:attachTrigger("tooltip.style.border.color", M.OnTooltipStyleBorderColor)
	LibEvent:attachTrigger("tooltip.style.mask", M.OnTooltipStyleMask)
	LibEvent:attachTrigger("tooltip:unit", M.OnTooltipUnit)
	self:CreateHooks()
	self:RemoveTooltip(NamePlateTooltip)
end

function M:OnEnable()
	if not IsAddOnLoaded("TinyTooltip") then
		self:RegisterEvent("ADDON_LOADED")
		return
	end

	self:CreateHooks()
	self:RemoveTooltip(NamePlateTooltip)
	self:StripGameTooltip()
end

function M:CreateHooks()
	if type(TinyTooltip) == "table" and type(TinyTooltip.MergeVariable) == "function" and not self:IsHooked(TinyTooltip, "MergeVariable") then
		self:RawHook(TinyTooltip, "MergeVariable", "TinyTooltip_MergeVariable")
	end
end

-- Remove a tooltip from the list of tooltips in TinyTooltip, if it exists.
function M:RemoveTooltip(tooltip)
	if TinyTooltip and TinyTooltip.tooltips then
		for i = #TinyTooltip.tooltips, 1, -1 do
			if TinyTooltip.tooltips[i] == tooltip then
				table.remove(TinyTooltip.tooltips, i)
			end
		end
	end
end

M:RemoveTooltip(NamePlateTooltip)

-- Strip the game tooltip's textures.
function M:StripGameTooltip()
	GameTooltip:DisableDrawLayer("BORDER")
	GameTooltip:DisableDrawLayer("BACKGROUND")
	GameTooltip.NineSlice:DisableDrawLayer("BORDER")
	GameTooltip.NineSlice:DisableDrawLayer("BACKGROUND")
	A:RegisterFrame("GameTooltipStatusBar"):Hide()
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:ADDON_LOADED(event, name)
	if name == "TinyTooltip" then
		self:UnregisterEvent("ADDON_LOADED")
		self:OnEnable()
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M.OnTooltipAnchor(_, tooltip)
	local hasMoved = M.HasGameTooltipDefaultContainerMoved()
	local relativeTo = hasMoved and GameTooltipDefaultContainer or SixUI_TooltipAnchor
	local point = hasMoved and A.GetScreenCorner(relativeTo) or "TOPLEFT"
	tooltip:ClearAllPoints()
	tooltip:SetPoint(point, relativeTo)
end

function M.OnTooltipHide(_, tooltip)
	B:RemoveBorder(tooltip.style)
end

function M.OnTooltipShow(_, tooltip)
	B:CreateBorder(tooltip.style)
end

function M.OnTooltipStyleBackground(_, tooltip, r, g, b, a)
	tooltip.style:SetBackdropColor(0, 0, 0, 0)
end

function M.OnTooltipStyleBorderColor(_, tooltip, r, g, b, a)
	local border = B:GetBorder(tooltip.style)
	local defaultColor = TinyTooltip.db.general.borderColor

	if border then
		if defaultColor[1] ~= r or defaultColor[1] ~= g or defaultColor[3] ~= b or defaultColor[4] ~= a then
			border:SetColor(r, g, b, a)
		else
			border:SetColor()
		end
	end

	tooltip.style:SetBackdropBorderColor(0, 0, 0, 0)
end

function M.OnTooltipStyleMask(_, tooltip)
	tooltip.style.mask:Hide()
end

function M.OnTooltipUnit(_, tooltip)
	tooltip.BigFactionIcon:Hide()
	tooltip.model:Hide()
end

function M:TinyTooltip_MergeVariable(TinyTooltip, src, dst)
	if src == TinyTooltip.db and dst == BigTipDB then
		src.general.mask = false
		src.item.coloredItemBorder = true
		src.unit.npc.showModel = false
		src.unit.player.elements.classIcon.enable = false
		src.unit.player.elements.factionBig.enable = false
		src.unit.player.elements.factionIcon.enable = false
		src.unit.player.elements.friendIcon.enable = false
		src.unit.player.elements.guildIndex.enable = true
		src.unit.player.elements.guildRealm.enable = false
		src.unit.player.elements.pvpIcon.enable = false
		src.unit.player.elements.realm.filter = "not samerealm"
		src.unit.player.elements.title.enable = false
		src.unit.player.showModel = false
	end

	return self.hooks[TinyTooltip].MergeVariable(TinyTooltip, src, dst)
end

M:CreateHooks()
